@@ -21,6 +21,7 @@ import (
 	"fmt"
 	"log"
 	"regexp"
+	"sort"
 	"strings"
 
 	"github.com/hashicorp/hcl/hcl/ast"
@@ -34,6 +35,25 @@ const safeChars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ012345678
 
 var unsafeChars = regexp.MustCompile(`[^0-9A-Za-z_\-]`)
 
+// make HCL output reproducible by sorting the AST nodes
+func sortHclTree(tree interface{}) {
+	switch t := tree.(type) {
+	case []*ast.ObjectItem:
+		sort.Slice(t, func(i, j int) bool {
+			var bI, bJ bytes.Buffer
+			_, _ = hclPrinter.Fprint(&bI, t[i]), hclPrinter.Fprint(&bJ, t[j])
+			return bI.String() < bJ.String()
+		})
+	case []ast.Node:
+		sort.Slice(t, func(i, j int) bool {
+			var bI, bJ bytes.Buffer
+			_, _ = hclPrinter.Fprint(&bI, t[i]), hclPrinter.Fprint(&bJ, t[j])
+			return bI.String() < bJ.String()
+		})
+	default:
+	}
+}
+
 // sanitizer fixes up an invalid HCL AST, as produced by the HCL parser for JSON
 type astSanitizer struct{}
 
@@ -44,6 +64,7 @@ func (v *astSanitizer) visit(n interface{}) {
 		v.visit(t.Node)
 	case *ast.ObjectList:
 		var index int
+		sortHclTree(t.Items)
 		for {
 			if index == len(t.Items) {
 				break
@@ -56,7 +77,9 @@ func (v *astSanitizer) visit(n interface{}) {
 		v.visitObjectItem(t)
 	case *ast.LiteralType:
 	case *ast.ListType:
+		sortHclTree(t.List)
 	case *ast.ObjectType:
+		sortHclTree(t.List)
 		v.visit(t.List)
 	default:
 		fmt.Printf(" unknown type: %T\n", n)
@@ -76,6 +99,9 @@ func (v *astSanitizer) visitObjectItem(o *ast.ObjectItem) {
 						break
 					}
 				}
+				if strings.HasPrefix(v, "--") { // if the key starts with "--", we must quote it. Seen in aws_glue_job.default_arguments parameter
+					v = fmt.Sprintf(`"%s"`, v)
+				}
 				if safe {
 					k.Token.Text = v
 				}
@@ -115,6 +141,8 @@ func (v *astSanitizer) visitObjectItem(o *ast.ObjectItem) {
 				}
 			}
 		}
+	case *ast.ListType:
+		sortHclTree(t.List)
 	default:
 	}
 
